==> sage-v0.3-2005-05-02-src.tar: sage-0.3/sage/functions/transcendental.py <==
"""
Computation of transcendental functions.
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import sage.pari.py_pari as py_pari
pari = py_pari.pari
import sage.rings.rings as rings
import sage.rings.complex_number

def __prep_num(x):
    if isinstance(x, sage.rings.complex_number.ComplexNumber):
        x = str(x).replace("i","I")
    else:
        x = str(x)
    return x

I = sage.rings.complex_field.ComplexField().gen(0)
def __eval(x):
    return eval(x)

def exponential_integral_1(x, n=0):
    """
    Returns the exponential integral E_1(x). If the optional argument
    n is given, computes list of the first n values of the exponential
    integral E_1(x*m).

    The exponential integral E_1(x) is

             E_1(x) = int_{x}^{\infty} e^{-t}/t dt

    INPUT:
        x -- a positive real number

        n -- (default: 0) a nonnegative integer; if nonzero,
             then return a list of values E_1(x*m) for
             m = 1,2,3,...,n.   This is useful, e.g., when
             computing derivatives of L-functions.

    OUTPUT:
        float -- if n is 0 (the default)
      or
        list -- list of floats if n > 0  

    EXAMPLES:
        >>> exponential_integral_1(2)
        0.048900510708061118
        >>> w = exponential_integral_1(2,4); w
        [0.048900510708061118, 0.0037793524098489067, 0.00036008245216265867, 3.76656228439249e-05]


    IMPLEMENTATION: We use the PARI C-library functions eint1 and
    veceint1.  
        
    REFERENCE: See page 262, Prop 5.6.12, of Cohen's book "A Course
    in Computational Algebraic Number Theory".

    REMARKS: When called with the optional argument n, the PARI
    C-library is fast for values of n up to some bound, then very
    very slow.  For example, if x=5, then the computation takes less
    than a second for n=800000, and takes "forever" for n=900000.
    
    """
    if n <= 0:
        return float(pari(x).eint1())
    else:
        return [float(z) for z in pari(x).eint1(n)]

def gamma(s):
    """
    Gamma function at s.
    """
    return rings.ComplexField()(pari(s).gamma())

def gamma_inc(s, t):
    """
    Incomplete Gamma function Gamma(s,t).

    WARNING: This function requires having Mathematica installed on
    your computer.  If you are reading this and know a reference for
    how to compute Gamma(s,t), let me know so I (or you) can implement
    it in SAGE.   (Pari has the incomplete Gamma function, but only
    for s and t real.)
    
    """
    try:
        import sage.interface.mathematica as M
        m = M.Math()
    except:
        raise RuntimeError, "You must have mathematica installed to use gamma_inc."
    t = "Gamma[%s, %s]*1.0"%(s,t)
    t = t.replace("e","*10^")
    s = m.cmd(t)
    return M.eval_num(s)
    

def zeta(s):
    """
    Riemann zeta function at s with s a real or complex number.
    EXAMPLES:
        >> zeta(2)
        1.6449340668482264364724151667
    """
    return pari(s).zeta().python()

def zeta_symmetric(s):
    """
    Completed function xi(s) that satisfies xi(s) = xi(1-s) and
    has zeros at the same points as the Riemann zeta function.

    More precisely, xi(s) = gamma(s/2 + 1) * (s-1) * pi^(-s/2) * zeta(s)
    EXAMPLES:
        >> zeta_symmetric(0.7)
        0.49758041465112690357779107524
        >> zeta_symmetric(1-0.7)
        0.49758041465112690357779107525

    REFERENCE:
      I copied the definition of xi from
        http://www.math.ubc.ca/~pugh/RiemannZeta/RiemannZetaLong.html
    """
    def gamma(n): return n.gamma()
    def zeta(s): return s.zeta()
    s = pari(s)
    if (s-1).abs() < pari('0.000000000001'):
        return 0.5
    pi = py_pari.pi()
    w = gamma(s/2 + 1) * (s-1) * pi**(-s/2) * zeta(s)
    return w.python()

def pi(prec=0):
    return py_pari.pi(prec).python()
    

# doctests
if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])

==> sage-v0.3-2005-06-02-src.tar: sage-0.3/sage/functions/transcendental.py <==
"""
Computation of transcendental functions.
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import sage.pari.py_pari as py_pari
pari = py_pari.pari
import sage.rings.rings
import sage.rings.complex_number

def __prep_num(x):
    if isinstance(x, sage.rings.complex_number.ComplexNumber):
        x = str(x).replace("i","I")
    else:
        x = str(x)
    return x

I = sage.rings.rings.ComplexField().gen(0)
def __eval(x):
    return eval(x)

def exponential_integral_1(x, n=0):
    """
    Returns the exponential integral E_1(x). If the optional argument
    n is given, computes list of the first n values of the exponential
    integral E_1(x*m).

    The exponential integral E_1(x) is

             E_1(x) = int_{x}^{\infty} e^{-t}/t dt

    INPUT:
        x -- a positive real number

        n -- (default: 0) a nonnegative integer; if nonzero,
             then return a list of values E_1(x*m) for
             m = 1,2,3,...,n.   This is useful, e.g., when
             computing derivatives of L-functions.

    OUTPUT:
        float -- if n is 0 (the default)
      or
        list -- list of floats if n > 0  

    EXAMPLES:
        >>> exponential_integral_1(2)
        0.048900510708061118
        >>> w = exponential_integral_1(2,4); w
        [0.048900510708061118, 0.0037793524098489067, 0.00036008245216265867, 3.76656228439249e-05]


    IMPLEMENTATION: We use the PARI C-library functions eint1 and
    veceint1.  
        
    REFERENCE: See page 262, Prop 5.6.12, of Cohen's book "A Course
    in Computational Algebraic Number Theory".

    REMARKS: When called with the optional argument n, the PARI
    C-library is fast for values of n up to some bound, then very
    very slow.  For example, if x=5, then the computation takes less
    than a second for n=800000, and takes "forever" for n=900000.
    
    """
    if n <= 0:
        return float(pari(x).eint1())
    else:
        return [float(z) for z in pari(x).eint1(n)]

def gamma(s):
    """
    Gamma function at s.
    """
    return sage.rings.rings.ComplexField()(pari(s).gamma())

def gamma_inc(s, t):
    """
    Incomplete Gamma function Gamma(s,t).

    WARNING: This function requires having Mathematica installed on
    your computer.  If you are reading this and know a reference for
    how to compute Gamma(s,t), let me know so I (or you) can implement
    it in SAGE.   (Pari has the incomplete Gamma function, but only
    for s and t real.)
    
    """
    try:
        import sage.interface.mathematica as M
        m = M.Math()
    except:
        raise RuntimeError, "You must have mathematica installed to use gamma_inc."
    t = "Gamma[%s, %s]*1.0"%(s,t)
    t = t.replace("e","*10^")
    s = m.cmd(t)
    return M.eval_num(s)
    

def zeta(s):
    """
    Riemann zeta function at s with s a real or complex number.
    EXAMPLES:
        >> zeta(2)
        1.6449340668482264364724151667
    """
    return pari(s).zeta().python()

def zeta_symmetric(s):
    """
    Completed function xi(s) that satisfies xi(s) = xi(1-s) and
    has zeros at the same points as the Riemann zeta function.

    More precisely, xi(s) = gamma(s/2 + 1) * (s-1) * pi^(-s/2) * zeta(s)
    EXAMPLES:
        >> zeta_symmetric(0.7)
        0.49758041465112690357779107524
        >> zeta_symmetric(1-0.7)
        0.49758041465112690357779107525

    REFERENCE:
      I copied the definition of xi from
        http://www.math.ubc.ca/~pugh/RiemannZeta/RiemannZetaLong.html
    """
    def gamma(n): return n.gamma()
    def zeta(s): return s.zeta()
    s = pari(s)
    if (s-1).abs() < pari('0.000000000001'):
        return 0.5
    pi = py_pari.pi()
    w = gamma(s/2 + 1) * (s-1) * pi**(-s/2) * zeta(s)
    return w.python()

def pi(prec=0):
    return py_pari.pi(prec).python()
    

# doctests
if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])
